(*
ocamlsvga - Objective Caml interface for the SVGA library
Copyright (C) 1997 Jean-Christophe FILLIATRE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License version 2, as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU Library General Public License version 2 for more details
(enclosed in the file LGPL).
*)

(* $Id: gl.mli,v 1.10 1997/05/26 21:58:28 jc Exp $ *)

type graphics_context

external currentcontext : unit -> graphics_context = "gl_currentcontext_stub"

external width : graphics_context -> int = "gl_width_stub"
external height : graphics_context -> int = "gl_height_stub"
external bytesperpixel : graphics_context -> int = "gl_bytesperpixel_stub"
external colors : graphics_context -> int = "gl_colors_stub"
external bitsperpixel : graphics_context -> int = "gl_bitsperpixel_stub"
external bytewidth : graphics_context -> int = "gl_bytewidth_stub"


(* Configuration *)

val null_graphics_context : graphics_context

external setcontextvga : Vga.mode -> unit = "gl_setcontextvga_stub"
external setcontextvgavirtual : Vga.mode -> unit = 
                                            "gl_setcontextvgavirtual_stub"
external setcontextvirtual : int -> int -> int -> int -> string -> unit =
                                            "gl_setcontextvirtual_stub"
external setcontextwidth : int -> unit = "gl_setcontextwidth_stub"
external setcontextheight : int -> unit = "gl_setcontextheight_stub"
external allocatecontext : unit -> graphics_context = "gl_allocatecontext_stub"
external setcontext : graphics_context -> unit = "gl_setcontext_stub"
external getcontext : graphics_context -> unit = "gl_getcontext_stub"
external freecontext : graphics_context -> unit = "gl_freecontext_stub"


(* Line drawing *)

external setpixel : int -> int -> int -> unit = "gl_setpixel_stub"
external setpixelrgb : int -> int -> int*int*int -> unit = 
                                                "gl_setpixelrgb_stub"
external getpixel : int -> int -> int = "gl_getpixel_stub"
external getpixelrgb : int -> int -> int*int*int = "gl_getpixelrgb_stub"
external rgbcolor : int -> int -> int -> int = "gl_rgbcolor_stub"
external hline : int -> int -> int -> int -> unit = "gl_hline_stub"
external line : int -> int -> int -> int -> int -> unit = "gl_line_stub"
external circle : int -> int -> int -> int -> unit = "gl_circle_stub"


(* Box (bitmap) functions *)

external fillbox : int -> int -> int -> int -> int -> unit
                                               = "gl_fillbox_stub"
external getbox : int -> int -> int -> int -> string -> unit
                                               = "gl_getbox_stub"
external putbox : int -> int -> int -> int -> string -> unit
                                               = "gl_putbox_stub"
external putboxpart : int -> int -> int -> int -> int -> int -> string 
        -> int -> int -> unit = "gl_putboxpart_stub" "gl_putboxpart_stub_nat"
external putboxmask : int -> int -> int -> int -> string -> unit
                                                = "gl_putboxmask_stub"
external copybox : int -> int -> int -> int -> int -> int -> unit 
                                    = "gl_copybox_stub" "gl_copybox_stub_nat"
external copyboxtocontext : int -> int -> int -> int -> graphics_context
                                -> int -> int -> unit 
		   = "gl_copyboxtocontext_stub" "gl_copyboxtocontext_stub_nat"
external copyboxfromcontext : graphics_context -> int -> int -> int -> int 
                               -> int -> int -> unit
	       = "gl_copyboxfromcontext_stub" "gl_copyboxfromcontext_stub_nat"
  
  (* The following functions only work in 256-color modes: *)

external compileboxmask : int -> int -> string -> string -> unit
                                                = "gl_compileboxmask_stub"
external compiledboxmasksize : int -> int -> string -> int 
                                                = "gl_compiledboxmasksize_stub"
external putboxmaskcompiled : int -> int -> int -> int -> string -> unit
                                                = "gl_putboxmaskcompiled_stub"


(* Miscellaneous *)

external clearscreen : int -> unit = "gl_clearscreen_stub"
external scalebox : int -> int -> string -> int -> int -> string -> unit 
    = "gl_scalebox_stub" "gl_scalebox_stub_nat"
external setdisplaystart : int -> int -> unit  = "gl_setdisplaystart_stub"
external enableclipping : unit -> unit = "gl_enableclipping_stub"
external setclippingwindow : int -> int -> int -> int -> unit 
    = "gl_setclippingwindow_stub"
external disableclipping : unit -> unit = "gl_disableclipping_stub"


(* Screen buffering *)

external copyscreen : graphics_context -> unit = "gl_copyscreen_stub"
external setscreenoffset : int -> unit = "gl_setscreenoffset_stub"

external enablepageflipping : graphics_context -> int
                            = "gl_enablepageflipping_stub"
        (* returns 3 if triple buffering is enabled, 2 is page flipping
	   is enabled and 0 otherwise. *)


(* Text *)

type writemode_flag = 
  | WRITEMODE_OVERWRITE
  | WRITEMODE_MASKED
  | FONT_EXPANDED
  | FONT_COMPRESSED

external expandfont : int -> int -> int -> string -> string -> unit
                    = "gl_expandfont_stub"
external setfont : int -> int -> string -> unit = "gl_setfont_stub"
external colorfont : int -> int -> int -> string -> unit = "gl_colorfont_stub"
external raw_setwritemode : int -> unit = "gl_setwritemode_stub"
val setwritemode : writemode_flag list -> unit
external write : int -> int -> string -> unit = "gl_write_stub"
external writen : int -> int -> int -> string -> unit = "gl_writen_stub"
external setfontcolors : int -> int -> unit = "gl_setfontcolors_stub"

external font8x8 : unit -> string = "gl_font8x8_stub" (* compressed 8x8 font *)


(* 256-color palette *)

type color = { red   : int ; 
	       green : int ;
	       blue  : int  }  (* 6-bit values *)

type palette = color array

external setpalettecolor : int -> int -> int -> int -> unit 
  = "gl_setpalettecolor_stub"
external getpalettecolor : int -> int * int * int = "gl_getpalettecolor_stub"
val setpalettecolors : int -> int -> palette -> unit
val getpalettecolors : int -> int -> palette
val setpalette : palette -> unit
val getpalette : unit -> palette
external setrgbpalette : unit -> unit = "gl_setrgbpalette_stub"


