(*
ocamlsvga - Objective Caml interface for the SVGA library
Copyright (C) 1997 Jean-Christophe FILLIATRE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License version 2, as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU Library General Public License version 2 for more details
(enclosed in the file LGPL).
*)

(* $Id: keyboard.mli,v 1.3 1997/05/26 21:58:29 jc Exp $ *)


val scancode_ESCAPE	        : int

val scancode_1		        : int
val scancode_2			: int
val scancode_3			: int
val scancode_4			: int
val scancode_5			: int
val scancode_6			: int
val scancode_7			: int
val scancode_8			: int
val scancode_9			: int
val scancode_0			: int

val scancode_MINUS		: int
val scancode_EQUAL		: int

val scancode_BACKSPACE		: int
val scancode_TAB		: int

val scancode_Q			: int
val scancode_W			: int
val scancode_E			: int
val scancode_R			: int
val scancode_T			: int
val scancode_Y			: int
val scancode_U			: int
val scancode_I			: int
val scancode_O			: int
val scancode_P			: int
val scancode_BRACKET_LEFT	: int
val scancode_BRACKET_RIGHT	: int

val scancode_ENTER		: int

val scancode_LEFTCONTROL	: int

val scancode_A			: int
val scancode_S			: int
val scancode_D			: int
val scancode_F			: int
val scancode_G			: int
val scancode_H			: int
val scancode_J			: int
val scancode_K			: int
val scancode_L			: int
val scancode_SEMICOLON		: int
val scancode_APOSTROPHE		: int
val scancode_GRAVE	        : int

val scancode_LEFTSHIFT		: int
val scancode_BACKSLASH		: int

val scancode_Z			: int
val scancode_X			: int
val scancode_C			: int
val scancode_V			: int
val scancode_B			: int
val scancode_N			: int
val scancode_M			: int
val scancode_COMMA		: int
val scancode_PERIOD		: int
val scancode_SLASH		: int

val scancode_RIGHTSHIFT		: int
val scancode_KEYPADMULTIPLY	: int

val scancode_LEFTALT		: int
val scancode_SPACE		: int
val scancode_CAPSLOCK		: int

val scancode_F1			: int
val scancode_F2			: int
val scancode_F3			: int
val scancode_F4			: int
val scancode_F5			: int
val scancode_F6		        : int
val scancode_F7			: int
val scancode_F8			: int
val scancode_F9			: int
val scancode_F10		: int

val scancode_NUMLOCK		: int
val scancode_SCROLLLOCK		: int

val scancode_KEYPAD7		: int
val scancode_CURSORUPLEFT	: int
val scancode_KEYPAD8		: int
val scancode_CURSORUP		: int
val scancode_KEYPAD9		: int
val scancode_CURSORUPRIGHT	: int
val scancode_KEYPADMINUS	: int
val scancode_KEYPAD4		: int
val scancode_CURSORLEFT		: int
val scancode_KEYPAD5		: int
val scancode_KEYPAD6		: int
val scancode_CURSORRIGHT	: int
val scancode_KEYPADPLUS		: int
val scancode_KEYPAD1		: int
val scancode_CURSORDOWNLEFT	: int
val scancode_KEYPAD2		: int
val scancode_CURSORDOWN		: int
val scancode_KEYPAD3		: int
val scancode_CURSORDOWNRIGHT	: int
val scancode_KEYPAD0		: int
val scancode_KEYPADPERIOD	: int

val scancode_LESS		: int

val scancode_F11		: int
val scancode_F12		: int

val scancode_KEYPADENTER	: int
val scancode_RIGHTCONTROL	: int
val scancode_CONTROL		: int
val scancode_KEYPADDIVIDE	: int
val scancode_PRINTSCREEN	: int
val scancode_RIGHTALT		: int
val scancode_BREAK		: int	(* Beware: is 119     *)
val scancode_BREAK_ALTERNATIVE	: int	(* on some keyboards! *)

val scancode_HOME		: int
val scancode_CURSORBLOCKUP	: int	(* Cursor key block *)
val scancode_PAGEUP		: int
val scancode_CURSORBLOCKLEFT	: int	(* Cursor key block *)
val scancode_CURSORBLOCKRIGHT	: int	(* Cursor key block *)
val scancode_END		: int
val scancode_CURSORBLOCKDOWN	: int	(* Cursor key block *)
val scancode_PAGEDOWN		: int
val scancode_INSERT		: int
val scancode_REMOVE		: int


(* Initialize keyboard handler (brings keyboard into RAW mode).
   Raises (Failure "Keyboard initialization failed") if unsuccessful. *)
external init : unit -> unit = "keyboard_init_stub"

(* Set event handler invoked by keyboard_update(). *)
type keyboard_handler = int -> bool -> unit
val seteventhandler : keyboard_handler -> unit
  
(* Return keyboard to normal state. *)
external close : unit -> unit = "keyboard_close_stub"

(* Read raw keyboard device and handle events. Returns [false] if no event. *)
external update : unit -> bool = "keyboard_update_stub"

(* Similar to keyboard_update, but wait for an event to happen. *)
(* [This doesn't seem to work very well...] *)
external waitforupdate : unit -> unit = "keyboard_waitforupdate_stub"

(* keyboard_init sets default event handler that keeps track of complete *)
(* keyboard state: *)

val key_NOTPRESSED : bool  (* = false *)
val key_PRESSED    : bool  (* = true  *)

(* Modes for translatekeys. *)
type translate =
  | TRANSLATE_CURSORKEYS 	(* Map cursor block to keypad cursor. *)
  | TRANSLATE_DIAGONAL 	        (* Map keypad diagonal to keypad cursor. *)
  | TRANSLATE_KEYPADENTER 	(* Map keypad enter to main enter key. *)
  | DONT_CATCH_CTRLC            (* Disable Crtl-C check. *)

(* Revert to default handler. *)
external setdefaulteventhandler : unit -> unit 
  = "keyboard_setdefaulteventhandler_stub"

(* Return pointer to buffer holding state of each key (scancode). *)
(* Value 1 corresponds to key that is pressed, 0 means not pressed. *)
external getstate : unit -> string = "keyboard_getstate_stub"

(* Force keyboard state to nothing pressed (all zeroes). *)
external clearstate : unit -> unit = "keyboard_clearstate_stub"

(* Let default handler translate cursor key block events to numeric keypad *)
(* cursor key events and other translations. *)
val translatekeys : translate list -> unit 

(* Return nonzero if key is depressed. *)
external keypressed : int -> bool = "keyboard_keypressed_stub"
