(*
ocamlsvga - Objective Caml interface for the SVGA library
Copyright (C) 1997 Jean-Christophe FILLIATRE

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License version 2, as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU Library General Public License version 2 for more details
(enclosed in the file LGPL).
*)

(* $Id: vga.ml,v 1.7 1997/05/26 21:58:30 jc Exp $ *)

(* svgalib version : 1.2.10 *)

(* initialization *)

external init : unit -> unit = "vga_init_stub"
external clear : unit -> unit = "vga_clear_stub"


(* SVGA modes *)

type mode = int
	

let mode_TEXT 	 	= 0
let mode_G320x200x16  	= 1
let mode_G640x200x16  	= 2
let mode_G640x350x16  	= 3
let mode_G640x480x16  	= 4
let mode_G320x200x256 	= 5
let mode_G320x240x256 	= 6
let mode_G320x400x256 	= 7
let mode_G360x480x256 	= 8
let mode_G640x480x2   	= 9

let mode_G640x480x256 	= 10
let mode_G800x600x256 	= 11
let mode_G1024x768x256 	= 12

let mode_G1280x1024x256 = 13	(* Additional modes. *)

let mode_G320x200x32K 	= 14
let mode_G320x200x64K 	= 15
let mode_G320x200x16M 	= 16
let mode_G640x480x32K 	= 17
let mode_G640x480x64K 	= 18
let mode_G640x480x16M 	= 19
let mode_G800x600x32K 	= 20
let mode_G800x600x64K 	= 21
let mode_G800x600x16M 	= 22
let mode_G1024x768x32K 	= 23
let mode_G1024x768x64K 	= 24
let mode_G1024x768x16M 	= 25
let mode_G1280x1024x32K = 26
let mode_G1280x1024x64K = 27
let mode_G1280x1024x16M = 28

let mode_G800x600x16	= 29
let mode_G1024x768x16 	= 30
let mode_G1280x1024x16 	= 31

let mode_G720x348x2 	= 32		(* Hercules emulation mode *)

let mode_G320x200x16M32 = 33		(* 32-bit per pixel modes. *)
let mode_G640x480x16M32 = 34
let mode_G800x600x16M32 = 35
let mode_G1024x768x16M32 = 36
let mode_G1280x1024x16M32 = 37

(* additional resolutions *)
let mode_G1152x864x16 	= 38
let mode_G1152x864x256 	= 39
let mode_G1152x864x32K 	= 40
let mode_G1152x864x64K 	= 41
let mode_G1152x864x16M 	= 42
let mode_G1152x864x16M32 = 43

let mode_G1600x1200x16 	= 44
let mode_G1600x1200x256 = 45
let mode_G1600x1200x32K = 46
let mode_G1600x1200x64K = 47
let mode_G1600x1200x16M = 48
let mode_G1600x1200x16M32 = 49


external lastmodenumber : unit -> mode = "vga_lastmodenumber_stub"

let mode___GLASTMODE = mode_G1600x1200x16M32
let mode_GLASTMODE   = lastmodenumber()


external getmodename : mode -> string = "vga_getmodename_stub"

let string_of_mode = getmodename

external getmodenumber : string -> mode = "vga_getmodenumber_stub"

let mode_of_string name =
  try
    getmodenumber name
  with 
    Invalid_argument "getmodenumber" ->
      raise (Invalid_argument "mode_of_string")

let string_of_mode = getmodename

external hasmode : mode -> bool = "vga_hasmode_stub"
external setmode : mode -> unit = "vga_setmode_stub"

external getdefaultmode : unit -> mode = "vga_getdefaultmode_stub"
external getcurrentmode : unit -> mode = "vga_getcurrentmode_stub"

type chipset = int

let chipset_UNDEFINED 	= 0
let chipset_VGA		= 1
let chipset_ET4000	= 2
let chipset_CIRRUS	= 3
let chipset_TVGA8900	= 4
let chipset_OAK		= 5
let chipset_EGA		= 6
let chipset_S3		= 7
let chipset_ET3000	= 8
let chipset_MACH32	= 9
let chipset_GVGA6400	= 10
let chipset_ARK		= 11
let chipset_ATI		= 12
let chipset_ALI		= 13

let string_of_chipset = function
  | 0  -> "UNDEFINED"
  | 1  -> "VGA"
  | 2  -> "ET4000"
  | 3  -> "CIRRUS"
  | 4  -> "TVGA8900"
  | 5  -> "OAK"
  | 6  -> "EGA"
  | 7  -> "S3"
  | 8  -> "ET3000"
  | 9  -> "MACH32"
  | 10 -> "GVGA6400"
  | 11 -> "ARK"
  | 12 -> "ATI"
  | 13 -> "ALI"
  | _  -> invalid_arg "string_of_chipset"

external getcurrentchipset : unit -> chipset = "vga_getcurrentchipset_stub"


external getxdim : unit -> int = "vga_getxdim_stub"
external getydim : unit -> int = "vga_getydim_stub"
external getcolors : unit -> int = "vga_getcolors_stub"


type blit_flag = 
    HAVE_BITBLIT
  | HAVE_FILLBLIT
  | HAVE_IMAGEBLIT
  | HAVE_HLINELISTBLIT
  | HAVE_BLITWAIT
;;

let string_of_blit_flag = function
    HAVE_BITBLIT        -> "HAVE_BITBLIT"
  | HAVE_FILLBLIT       -> "HAVE_FILLBLIT"
  | HAVE_IMAGEBLIT      -> "HAVE_IMAGEBLIT"  
  | HAVE_HLINELISTBLIT  -> "HAVE_HLINELISTBLIT"
  | HAVE_BLITWAIT       -> "HAVE_BLITWAIT"
;;

let blit_flags_table = [|
    HAVE_BITBLIT
  ; HAVE_FILLBLIT
  ; HAVE_IMAGEBLIT
  ; HAVE_HLINELISTBLIT
  ; HAVE_BLITWAIT |]
;;

(* [blit_flags_of_int f] convert packed blit flags in integer f in a list of
   blit_flags. Optimised using blit_flags_table. *)
let blit_flags_of_int f = 
  let rec convert i f =
    if i = 5 then
      [] 
    else
      if f land 1 > 0 then 
	(blit_flags_table.(i))::(convert (succ i) (f/2))
      else
	convert (succ i) (f/2)
  in convert 0 f
;;

let _ = Callback.register "convert_blit_flags" blit_flags_of_int

type other_flag =
    HAVE_RWPAGE
  | IS_INTERLACED
  | IS_MODEX
  | IS_DYNAMICMODE
  | CAPABLE_LINEAR
  | IS_LINEAR
  | EXT_INFO_AVAILABLE
  | RGB_MISORDERED
  | HAVE_EXT_SET
;;

let string_of_other_flag = function
    HAVE_RWPAGE         -> "HAVE_RWPAGE"
  | IS_INTERLACED       -> "IS_INTERLACED"
  | IS_MODEX            -> "IS_MODEX"
  | IS_DYNAMICMODE      -> "IS_DYNAMICMODE"
  | CAPABLE_LINEAR      -> "CAPABLE_LINEAR"
  | IS_LINEAR           -> "IS_LINEAR"
  | EXT_INFO_AVAILABLE  -> "EXT_INFO_AVAILABLE"
  | RGB_MISORDERED      -> "RGB_MISORDERED"
  | HAVE_EXT_SET        -> "HAVE_EXT_SET"
;;

let other_flags_table = [|
    HAVE_RWPAGE
  ; IS_INTERLACED
  ; IS_MODEX
  ; IS_DYNAMICMODE
  ; CAPABLE_LINEAR
  ; IS_LINEAR
  ; EXT_INFO_AVAILABLE
  ; RGB_MISORDERED
  ; HAVE_EXT_SET  |]
;;

(* [other_flags_of_int f] convert packed other flags in integer f in a list of
   other_flags. Optimised using other_flags_table. *)
let other_flags_of_int f = 
  let rec convert i f =
    if i = 9 then
      [] 
    else
      if f land 1 > 0 then 
	(other_flags_table.(i))::(convert (succ i) (f/2))
      else
	convert (succ i) (f/2)
  in convert 0 f
;;

let _ = Callback.register "convert_other_flags" other_flags_of_int

type modeinfo = {
  width : int ;
  height : int ;
  bytesperpixel : int ;
  colors : int ;
  linewidth : int ;
  maxlogicalwidth : int ;
  startaddressrange : int ;
  maxpixels : int ;
  haveblit : blit_flag list ;
  flags : other_flag list ;
  chiptype : chipset ;
  memory : int ;
  linewidth_unit : int }
;;

external getmodeinfo : mode -> modeinfo = "vga_getmodeinfo_stub"


(* functions to set some of the chipset parameters *)

external setpage : int -> unit = "vga_setpage_stub"
external setreadpage : int -> unit = "vga_setreadpage_stub"
external setwritepage : int -> unit = "vga_setwritepage_stub"
external setlogicalwidth : int -> unit = "vga_setlogicalwidth_stub"
external setdisplaystart : int -> unit = "vga_setdisplaystart_stub"

external claimvideomemory : int -> unit = "vga_claimvideomemory_stub"

external setmodeX : unit -> unit = "vga_setmodeX_stub"
external disabledriverreport : unit -> unit = "vga_disabledriverreport_stub"
external getmousetype : unit -> int = "vga_getmousetype_stub"
external getmonitortype : unit -> int = "vga_getmonitortype_stub"


(* refresh on/off *)

external screenoff : unit -> unit = "vga_screenoff_stub"
external screenon : unit -> unit = "vga_screenon_stub"
external waitretrace : unit -> unit = "vga_waitretrace_stub"


(* flipping *)

external setflipchar : int -> unit = "vga_setflipchar_stub"
external flip : unit -> unit = "vga_flip_stub"
external getch : unit -> int = "vga_getch_stub"


(* drawing functions *)

external setcolor : int -> unit = "vga_setcolor_stub"
external drawpixel : int -> int -> unit = "vga_drawpixel_stub"
external drawline : int -> int -> int -> int -> unit = "vga_drawline_stub"

external getpixel : int -> int -> int = "vga_getpixel_stub"

external drawscanline : int -> string -> unit = "vga_drawscanline_stub"
external drawscansegment : string -> int -> int -> int -> unit
				    	      = "vga_drawscansegment_stub"
external getscansegment : string -> int -> int -> int -> unit
					      = "vga_getscansegment_stub"


(* palette *)

external setpalette : int -> int -> int -> int -> unit = "vga_setpalette_stub"
external getpalette : int -> int * int * int = "vga_getpalette_stub"
external setpalvec : int -> int -> int array -> unit = "vga_setpalvec_stub"
external getpalvec : int -> int -> int array -> unit = "vga_getpalvec_stub"
external white : unit -> int = "vga_white_stub"
external setegacolor : int -> unit = "vga_setegacolor_stub"
external setrgbcolor : int -> int -> int -> unit = "vga_setrgbcolor_stub"


(* keyboard *)

external getkey : unit -> int = "vga_getkey_stub"


(* dump the VGA registers *)

external dumpregs : unit -> unit = "vga_dumpregs_stub"
